'' =================================================================================================
''
''   File....... jm_pan-tilt_demo.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2013 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000

  PWM_FREQ = 20_000                                             ' run motors @ 20kHz


con

  RX1     = 31                                                  ' programming / terminal
  TX1     = 30
  
  SDA     = 29                                                  ' eeprom / i2c
  SCL     = 28

  ADC_CS  = 25
  ADC_CLK = 24
  ADC_DIO = 23 
                           
  M1_EN   =  5 
  M1_DIR  =  4
  M1_SPD  =  3 
         
  M0_EN   =  2
  M0_DIR  =  1
  M0_SPD  =  0


con

  SAMPLES =  8
  

con

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR       ' PST formmatting control
  #14, GOTOX, GOTOY, CLS


obj

  term   : "fullduplexserial64"                                 ' uses cog
  adc    : "jm_mcp3202_ez"
  motors : "jm_dualmotor_ez"                                    ' uses cog
 

var

  long  xraw[SAMPLES]
  long  xavg
  long  xspeed
  
  long  yraw[SAMPLES]
  long  yavg
  long  yspeed

  long  printflag

  long  monstack[32]                                            ' for monitor cog


pub main | t, idx1, idx2, xwork, ywork

  term.start(RX1, TX1, %0000, 115_200)                          ' start serial cog
  cognew(monitor, @monstack)                                    ' start monitor cog

  adc.start(ADC_CS, ADC_CLK, ADC_DIO)                           ' start mcp3202 object 

  motors.start(M0_SPD, M0_DIR, M1_SPD, M1_DIR, PWM_FREQ)        ' start motors pwm cog
  high(M0_EN)                                                   ' enable motors
  high(M1_EN)

  pause(50)

  idx1 := 0
  
  t := cnt
  repeat
    xraw[idx1] := adc.read(0, adc#SE) >> 2                      ' get new reading
    yraw[idx1] := adc.read(1, adc#SE) >> 2

    xwork := 0                                                  ' clear workspace vars
    ywork := 0
    repeat idx2 from 0 to SAMPLES-1                             ' accumulate readings
      xwork += xraw[idx2]
      ywork += yraw[idx2]

    xavg := xwork / SAMPLES                                     ' average
    xspeed := scale(xwork, 1023, -100, 100)                     ' convert to motor speed
    motors.setspeed(0, xspeed)  

    yavg := ywork / SAMPLES
    yspeed := scale(ywork, 1023, -100, 100)
    motors.setspeed(1, yspeed)

    printflag := true                                           ' update monitor screen  
          
    idx1 := ++idx1 // SAMPLES                                   ' update samples index    

    waitcnt(t += (MS_001 * 50))                                 ' run loop every 50ms


pub scale(raw, range, minout, maxout)

'' Scales raw (0 to range) value to new range: minout to maxout

  raw := 0 #> raw <# range                                      ' constrain input

  if (minout < maxout)                                          ' if good range
    return ((raw * (maxout - minout)) / range) + minout
  else
    return raw
    

con

  { --------------- }
  {                 }
  {   B A S I C S   }
  {                 }
  { --------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]
  

con

  { ----------------- }
  {                   }
  {   M O N I T O R   }
  {                   } 
  { ----------------- }

  
pri monitor

  term.tx(CLS)
  term.str(string("Joystick / Motor Controller"))

  repeat
    term.tx(HOME)
    term.tx(LF)
    term.tx(LF)

    repeat until (printflag)                                    ' wait for new values
    printflag := false                                          ' reset

    term.str(string("0: "))
    term.rjdec(xavg, 4, " ")
    term.rjdec(xspeed, 6, " ")
    term.tx(CR)

    term.str(string("1: "))
    term.rjdec(yavg, 4, " ")
    term.rjdec(yspeed, 6, " ") 


dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}